(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (factory((global.filtry = {})));
}(this, (function (exports) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    var __assign = function() {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };

    var buildTree = function (object, id, childrenKey) {
        if (childrenKey === void 0) { childrenKey = 'equations'; }
        // @ts-ignore
        var res = __assign({}, object[id]);
        if (res[childrenKey]) {
            var newChildren = {};
            for (var _i = 0, _a = res[childrenKey]; _i < _a.length; _i++) {
                var childId = _a[_i];
                // @ts-ignore
                newChildren[childId] = buildTree(object, childId);
            }
            res[childrenKey] = newChildren;
        }
        return res;
    };

    var filterRowByEquation = function (filterNode, row, getData) {
        if (filterNode.type === 'equation') {
            var isDate = filterNode.value instanceof Date;
            if (getData) {
                row = getData(row);
            }
            if (filterNode.key === null || filterNode.op === null) {
                return false;
            }
            // @ts-ignore
            var rowValue = row[filterNode.key];
            switch (filterNode.op) {
                case '===': {
                    if (isDate) {
                        if (!rowValue && filterNode.value) {
                            return false;
                        }
                        if (!filterNode.value) {
                            return true;
                        }
                        return rowValue.getTime() === filterNode.value.getTime();
                    }
                    return rowValue === filterNode.value;
                }
                case '!==': {
                    return rowValue !== filterNode.value;
                }
                case 'isnull': {
                    return rowValue == null;
                }
                case '!isnull': {
                    return rowValue != null;
                }
                case '<': {
                    return rowValue < filterNode.value;
                }
                case '>': {
                    return rowValue > filterNode.value;
                }
                case '<=': {
                    return rowValue <= filterNode.value;
                }
                case '>=': {
                    return rowValue >= filterNode.value;
                }
                case 'contains': {
                    if (rowValue == null || typeof rowValue.includes !== 'function') {
                        return false;
                    }
                    return rowValue.includes(filterNode.value);
                }
                case '!contains': {
                    if (rowValue == null || typeof rowValue.includes !== 'function') {
                        return true;
                    }
                    return !rowValue.includes(filterNode.value);
                }
                case 'in': {
                    if (filterNode.value == null || typeof filterNode.value.includes !== 'function') {
                        return false;
                    }
                    return filterNode.value.includes(rowValue);
                }
                case '!in': {
                    if (filterNode.value == null || typeof filterNode.value.includes !== 'function') {
                        return true;
                    }
                    return !filterNode.value.includes(rowValue);
                }
            }
            return false;
        }
        else if (filterNode.type === 'and') {
            for (var _i = 0, _a = Object.values(filterNode.equations); _i < _a.length; _i++) {
                var equation = _a[_i];
                if (!filterRowByEquation(equation, row, getData)) {
                    return false;
                }
            }
            return true;
        }
        else if (filterNode.type === 'or') {
            for (var _b = 0, _c = Object.values(filterNode.equations); _b < _c.length; _b++) {
                var equation = _c[_b];
                if (filterRowByEquation(equation, row, getData)) {
                    return true;
                }
            }
            return false;
        }
        throw new Error('Unsupported equation type given: ' + filterNode);
    };
    var buildFilterTree = function (filter) {
        return buildTree(filter.equations, filter.root);
    };
    var filter = function (data, filter, getData) {
        if (getData === void 0) { getData = null; }
        if (filter.root === null || filter.root === undefined) {
            return data;
        }
        var filterTree = buildFilterTree(filter);
        var filterFunction = function (row) { return filterRowByEquation(filterTree, row, getData); };
        return data.filter(filterFunction);
    };

    exports.filter = filter;
    exports.buildFilterTree = buildFilterTree;
    exports.default = filter;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=filtry.umd.js.map
