"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var filterRowByEquation = function (filterNode, row, getData) {
    if (filterNode.type === 'equation') {
        var isDate = filterNode.value instanceof Date;
        if (getData) {
            row = getData(row);
        }
        if (filterNode.key === null || filterNode.op === null) {
            return false;
        }
        // @ts-ignore
        var rowValue = row[filterNode.key];
        switch (filterNode.op) {
            case '===': {
                if (isDate) {
                    if (!rowValue && filterNode.value) {
                        return false;
                    }
                    if (!filterNode.value) {
                        return true;
                    }
                    return rowValue.getTime() === filterNode.value.getTime();
                }
                return rowValue === filterNode.value;
            }
            case '!==': {
                return rowValue !== filterNode.value;
            }
            case 'isnull': {
                return rowValue == null;
            }
            case '!isnull': {
                return rowValue != null;
            }
            case '<': {
                return rowValue < filterNode.value;
            }
            case '>': {
                return rowValue > filterNode.value;
            }
            case '<=': {
                return rowValue <= filterNode.value;
            }
            case '>=': {
                return rowValue >= filterNode.value;
            }
            case 'contains': {
                if (rowValue == null || typeof rowValue.includes !== 'function') {
                    return false;
                }
                return rowValue.includes(filterNode.value);
            }
            case '!contains': {
                if (rowValue == null || typeof rowValue.includes !== 'function') {
                    return true;
                }
                return !rowValue.includes(filterNode.value);
            }
            case 'in': {
                if (filterNode.value == null || typeof filterNode.value.includes !== 'function') {
                    return false;
                }
                return filterNode.value.includes(rowValue);
            }
            case '!in': {
                if (filterNode.value == null || typeof filterNode.value.includes !== 'function') {
                    return true;
                }
                return !filterNode.value.includes(rowValue);
            }
        }
        return false;
    }
    else if (filterNode.type === 'and') {
        for (var _i = 0, _a = Object.values(filterNode.equations); _i < _a.length; _i++) {
            var equation = _a[_i];
            if (!filterRowByEquation(equation, row, getData)) {
                return false;
            }
        }
        return true;
    }
    else if (filterNode.type === 'or') {
        for (var _b = 0, _c = Object.values(filterNode.equations); _b < _c.length; _b++) {
            var equation = _c[_b];
            if (filterRowByEquation(equation, row, getData)) {
                return true;
            }
        }
        return false;
    }
    throw new Error('Unsupported equation type given: ' + filterNode);
};
var buildFilterTree = function (filter) {
    return utils_1.buildTree(filter.equations, filter.root);
};
exports.buildFilterTree = buildFilterTree;
var filter = function (data, filter, getData) {
    if (getData === void 0) { getData = null; }
    if (filter.root === null || filter.root === undefined) {
        return data;
    }
    var filterTree = buildFilterTree(filter);
    var filterFunction = function (row) { return filterRowByEquation(filterTree, row, getData); };
    return data.filter(filterFunction);
};
exports.filter = filter;
exports.default = filter;
//# sourceMappingURL=filtry.js.map